"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogLevel = exports.typescriptCompileMiddleware = void 0;
var tslib_1 = require("tslib");
var express_1 = require("express");
var create_compiler_options_1 = require("./lib/create-compiler-options");
var create_resolver_1 = require("./lib/create-resolver");
var create_source_code_provider_1 = require("./lib/create-source-code-provider");
var create_transpiler_1 = require("./lib/create-transpiler");
var logger_1 = require("./lib/logger");
Object.defineProperty(exports, "LogLevel", { enumerable: true, get: function () { return logger_1.LogLevel; } });
var modules_preload_handle_1 = require("./lib/modules-preload-handle");
var options_1 = require("./lib/options");
var modules_import_handler_1 = require("./lib/modules-import-handler");
tslib_1.__exportStar(require("./lib/utils/get-url-for-file"), exports);
/**
 * Creates instance of typescript compile middleware.
 * @public
 * @param options - options of middleware
 * @param logger - logging target, default `console`
 */
function typescriptCompileMiddleware(options, logger) {
    if (options === void 0) { options = {}; }
    var preloadList = [];
    var config = (0, options_1.prepareConfig)(options);
    var cwd = config.cwd, compile = config.compile, resolve = config.resolve, cache = config.cache, logLevel = config.logLevel;
    var log = (0, logger_1.createLogger)(logLevel, logger);
    var compilerOptions = (0, create_compiler_options_1.createCompilerOptions)(cwd, compile, log);
    var resolver = (0, create_resolver_1.createResolver)(cwd, compilerOptions, resolve, log);
    var transpiler = (0, create_transpiler_1.createTranspiler)(cwd, compilerOptions, compile.transformers || [], resolver, preloadList);
    var sourceProvider = (0, create_source_code_provider_1.createSourceCodeProvider)(cwd, cache, transpiler);
    var router = (0, express_1.Router)({ mergeParams: true });
    router.get('/modules-preload', (0, modules_preload_handle_1.modulesPreloadHandle)(preloadList));
    router.get('*', (0, modules_import_handler_1.modulesImportHandler)(config, sourceProvider, log));
    return router;
}
exports.typescriptCompileMiddleware = typescriptCompileMiddleware;
//# sourceMappingURL=index.js.map