"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createImportsTransformer = void 0;
var tslib_1 = require("tslib");
var typescript_1 = require("typescript");
var get_url_for_file_1 = require("./utils/get-url-for-file");
/**
 * Create source code transformer
 * @internal
 * @param cwd - working dir
 * @param resolve - dependencies resolver
 * @param file - target file
 * @param preloadList - list modules to preload
 */
function createImportsTransformer(cwd, resolve, file, preloadList) {
    var checkIgnore = function (node) {
        var _a;
        var tags = (0, typescript_1.getJSDocTags)(node);
        for (var _i = 0, tags_1 = tags; _i < tags_1.length; _i++) {
            var tag = tags_1[_i];
            if (((_a = tag === null || tag === void 0 ? void 0 : tag.tagName) === null || _a === void 0 ? void 0 : _a.escapedText) === 'etc-ignore') {
                return true;
            }
        }
        return false;
    };
    /**
     * Creates import url.
     * @param target - for the dependency
     */
    var url = function (target) {
        var url;
        var resolution = resolve(file, target);
        if (resolution.isExternal) {
            url = resolution.resolution;
        }
        else if (resolution.isUrl) {
            url = resolution.resolution;
        }
        else {
            url = (0, get_url_for_file_1.getUrlForFile)(cwd, resolution.resolution);
        }
        if (preloadList.indexOf(url) < 0) {
            preloadList.push(url);
        }
        return url;
    };
    return function (context) {
        // creates visitor
        var visitor = function (node) {
            var _a;
            // dynamic import call
            if ((0, typescript_1.isCallExpression)(node) && node.expression.kind === typescript_1.SyntaxKind.ImportKeyword) {
                if (checkIgnore(node)) {
                    return undefined;
                }
                var _b = node.arguments, arg = _b[0], rest = _b.slice(1);
                if ((0, typescript_1.isStringLiteral)(arg)) {
                    return context.factory.updateCallExpression(node, node.expression, node.typeArguments, tslib_1.__spreadArray([
                        context.factory.createStringLiteral(url(arg.text))
                    ], rest, true));
                }
                // todo report problematic import
            }
            // export ... from ...
            if ((0, typescript_1.isExportDeclaration)(node) && !node.isTypeOnly) {
                if (checkIgnore(node)) {
                    return undefined;
                }
                var moduleSpecifier = node.moduleSpecifier;
                if (moduleSpecifier && (0, typescript_1.isStringLiteral)(moduleSpecifier)) {
                    return context.factory.updateExportDeclaration(node, node.modifiers, node.isTypeOnly, node.exportClause, context.factory.createStringLiteral(url(moduleSpecifier.text)), node.assertClause);
                }
            }
            // import ... from ...
            if ((0, typescript_1.isImportDeclaration)(node) && !((_a = node.importClause) === null || _a === void 0 ? void 0 : _a.isTypeOnly)) {
                if (checkIgnore(node)) {
                    return undefined;
                }
                var moduleSpecifier = node.moduleSpecifier;
                if ((0, typescript_1.isStringLiteral)(moduleSpecifier)) {
                    return context.factory.updateImportDeclaration(node, node.modifiers, node.importClause, context.factory.createStringLiteral(url(moduleSpecifier.text)), node.assertClause);
                }
                return node;
            }
            return (0, typescript_1.visitEachChild)(node, visitor, context);
        };
        return function (node) {
            return (0, typescript_1.visitEachChild)(node, visitor, context);
        };
    };
}
exports.createImportsTransformer = createImportsTransformer;
//# sourceMappingURL=create-imports-transformer.js.map